/* 
 *
 * PDFlib PLOP: PDF Linearization, Optimization, Protection
 * certsecertsec sample
 *
 */

#include <iostream>
#include <string>

#include "plop.hpp"

using namespace std;
using namespace pdflib;

namespace
{
wstring get_wstring(const PLOP& plop, const string& utf8_string);
}

int
main(int argc, char **argv)
{
    PLOP plop;

    /* This is where input files live. Adjust as necessary. */
    const wstring searchpath(L"../data");

    /* Recipient certificates and permissions */
    struct recipientdata {
        recipientdata(wstring c, wstring p):
          cert(c), perm(p) {}
        wstring cert;
        wstring perm;
    };

    const recipientdata recipients[] = {
        /* no permission restrictions */
        recipientdata(L"../data/demo_recipient_1.pem", L""),

        /* form filling and signing permissions granted, but not others */
        recipientdata(L"../data/demo_recipient_2.pem", L"nomodify nocopy noannots noassemble"),
        recipientdata(L"../data/demo_recipient_3.pem", L"nomodify nocopy noannots noassemble"),
    };
    const int RECIPIENTCOUNT = sizeof(recipients) / sizeof(recipients[0]);


    if (argc < 3)
    {
        wcerr << L"usage: sign <infile> <outfile>" << endl;
        return 2;
    }

    try {
        int doc;
        int r;
        wostringstream optlist;

        optlist << L"searchpath={" << searchpath << L"}";
        plop.set_option(optlist.str());

        /*
         * Caution: For simplicity we assume that the program arguments are
         * encoded as UTF-8, which might not be true in all cases!
         */
        const wstring doc_name(get_wstring(plop, string(argv[1])));
        if ((doc = plop.open_document(doc_name, L"")) == -1)
        {
            wcerr << L"Error: " << plop.get_apiname() << L": "
                                << plop.get_errmsg() << endl;
            return 2;
        }

        /* add recipients */
        for (r = 0; r < (int)RECIPIENTCOUNT; r++)
        {
            optlist.str(L"");
            optlist << L"certificate={filename={";
            optlist << recipients[r].cert;
            optlist << "}} permissions={";
            optlist << recipients[r].perm;
            optlist << "}";

            if (plop.add_recipient(optlist.str()) == -1)
            {
                /* emit warning and continue */
                wcerr << L"Warning: " << plop.get_errmsg() << endl;
            }
        }


        /* create protected output file */
        optlist.str(L"");
        optlist << L"input=" << doc;

        const wstring out_name(get_wstring(plop, string(argv[2])));
        if (plop.create_document(out_name, optlist.str()) == -1)
        {
            wcerr << L"Error: " << plop.get_errmsg() << endl;
            return 2;
        }

        /* close input and output files */
        plop.close_document(doc, L"");
    }
    catch (PLOP::Exception &ex) {
        wcerr << L"PLOP exception occurred in certsec sample:" << endl;
        wcerr << L"[" << ex.get_errnum() << L"] " << ex.get_apiname()
                << L": " << ex.get_errmsg() << endl;
        return 2;
    }

    return 0;
}

namespace
{

/*
 * Get a wstring for the given string, assuming the string is encoded in
 * UTF-8.
 */
wstring get_wstring(const PLOP& plop, const string& utf8_string)
{
    const size_t size = sizeof(wstring::value_type);
    string wide_string;

    switch (size)
    {
    case 2:
        wide_string = plop.convert_to_unicode(L"utf8",
                            utf8_string, L"outputformat=utf16");
        break;

    case 4:
        wide_string = plop.convert_to_unicode(L"utf8",
                            utf8_string, L"outputformat=utf32");
        break;

    default:
        throw std::logic_error("Unsupported wchar_t size");
    }

    return wstring(reinterpret_cast<const wchar_t *>(wide_string.data()),
            wide_string.length() / size);
}

} // end anonymous namespace

