<?php
/*
 * PDFlib PLOP DS:
 * generate a dynamic visualization document in memory with PDFlib,
 * and digitally sign a second document, using the dynamic document
 * for signature visualization.
*/

date_default_timezone_set('Europe/Berlin');
# This is where input files live. Adjust as necessary.
$searchpath = "{../data} {../../data}";

$months = array( "January", "February", "March", "April", "May", "June",
            "July", "August", "September", "October", "November", "December");


# Basic signature options
$sign_opts =
    "ltv=try " .        /* use ltv=full to ensure LTV status */
    /* use the following line for PKCS11 usage
    "engine=pkcs#11 digitalid={filename=cryptoki.dll} "
    */
    "engine=builtin digitalid={filename=demo_signer_rsa_2048.p12} password={demo} " ;

$infile = "PLOP-datasheet.pdf";

# -------- Step 1: create visualization document in memory  with PDFlib -------

try {
    $p = new PDFlib();

    # create a new PDF document in memory

    $p->begin_document("", "");

    $p->set_info("Creator", "dynamicsign.php");
    $p->set_info("Author", "Rainer Pl�ckl");
    $p->set_info("Title", "Dynamic signature visualization document");

    $p->begin_page_ext(300, 50, "");
    # Prepare dynamic text for the signature visualization
    $timebuf = sprintf("Digitally signed %s %d, %d at %02d:%02d:%02d",
             $months[localtime()[4]], localtime()[3], localtime()[5]+1900,
            localtime()[2], localtime()[1], localtime()[0]);

     $textoptlist =sprintf( "fontsize=10 fontname=Helvetica " .
            "encoding=unicode textformat=utf8 fillcolor=red");
    $p->fit_textline($timebuf, 10, 10, $textoptlist);
    
    $p->end_page_ext("");

    $p->end_document("");

    # fetch the memory buffer containing the PDF document
    $contents = $p->get_buffer();

# ------------------ Step 2: sign the document -----------------
    
    $plop = new PLOP();


    $plop->set_option("searchpath={{$searchpath}}");


    # Create a virtual PVF file from the document data in memory
    $plop->create_pvf("/pvf/visdoc.pdf", $contents, "");

    # open unsigned input document from disc
    if (($doc = $plop->open_document($infile, "")) == 0)
    {
        die("Error: " . $plop->get_apiname() . ": "
                                 . $plop->get_errmsg() . "\n");
    }

    # open unsigned input document from PVF file
    if (($visdoc = $plop->open_document("/pvf/visdoc.pdf", "")) == 0)
    {
        die("Error: " . $plop->get_apiname() . ": "
                                 . $plop->get_errmsg() . "\n");
    }
    # Add field option for visualization to base signature options
    $optlist = sprintf("%s field={visdoc=%d rect={10 10 adapt adapt}} ",
                    $sign_opts, $visdoc);

    if ($plop->prepare_signature( $optlist) == 0)
    {
        die("Error: " . $plop->get_apiname() . ": "
                                 . $plop->get_errmsg() . "\n");
    }

    # sign the output file
    if ($plop->create_document("", "input=" . $doc) == 0)
    {
        die("Error: " . $plop->get_apiname() . ": "
                                 . $plop->get_errmsg() . "\n");
    }

    $buf = $plop->get_buffer();
    $len = strlen($buf);

    header("Content-type: application/pdf");
    header("Content-Length: $len");
    header("Content-Disposition: inline; filename=dynamicsign.pdf");
    print $buf;

    # close the input document
    $plop->close_document($doc, "");

    # close the visualization document
    $plop->close_document($visdoc, "");

    # delete PVF file containing input document
    if ($plop->delete_pvf("/pvf/visdoc.pdf") == 0)
    {
        die("Error: %s\n" .  $plop->get_errmsg());
    }

}
catch (PLOPException $e) {
    die("PLOP exception occurred in dynamicsign sample:\n" .
        "[" . $e->get_errnum() . "] " . $e->get_apiname() . ": " .
        $e->get_errmsg() . "\n");
    }
catch (Exception $e) {
    die($e);
}

$p = 0;

