#!/usr/bin/perl
# Id: hellosign.pl,v 1.1 2011/02/05 21:09:49 rjs Exp 
#
# PDFlib PLOP DS:
# generate a document in memory with PDFlib and sign it with PLOP DS
#

from sys import *
from traceback import *
from PDFlib.PLOP import *

from PDFlib.PDFlib import *

# Basic signature options

# use ltv=full to ensure LTV status
# for  use with PKCS11
# "engine=pkcs#11 digitalid={filename=cryptoki.dll} "
sign_opts = \
    "ltv=try " \
    "engine=builtin digitalid={filename=demo_signer_rsa_2048.p12} password={demo} "

# This is where input files live. Adjust as necessary.
searchpath = "../data"

out_filename = "hellosign.pdf"

plop = None
p = None
# ----------  Step 1: generate document in memory with PDFlib ---------- 
try:
    try:
        # create a new PDFlib object
        p = PDFlib()

        # create a new PDF document in memory
        p.begin_document("", "")

        p.set_info("Creator", "hellosign.py")
        p.set_info("Author", "Rainer Ploeckl")
        p.set_info("Title", "Hello, world/signed (Python)!")

        font = p.load_font("Helvetica-Bold", "host", "")

        p.begin_page_ext(595, 842, "")

        p.setfont(font, 24)
        p.set_text_pos(50, 700)
        p.show("Hello, world/signed!")
        p.continue_text("(says Python)")
        p.end_page_ext("")

        p.end_document("")

        # fetch the buffer containing the PDF document
        contents = p.get_buffer()

    except PDFlibException:
        print_exc(file=stdout)

except Exception:
    print_exc(file=stdout)

# ------------------ Step 2: sign the document -----------------
    
try:
    try:
        # create a new PLOP object
        plop = PLOP()

        optlist = "searchpath={%s}" % searchpath
        plop.set_option(optlist)

        # Create a virtual PVF file from the document data in memory
        plop.create_pvf("/pvf/hello.pdf", contents, "")

        # open input document from PVF file
        doc = plop.open_document("/pvf/hello.pdf", "")
        if (doc == -1):
            raise PLOPException("Error: %s" % (PLOP_get_errmsg(),
                                                     plop.get_errmsg()))

        # sign the output file
        if (plop.prepare_signature(sign_opts) == -1):
            raise PLOPException("Error: %s: %s" % (plop.get_apiname(),
                                                     plop.get_errmsg()))

        optlist = "input={%d}" % doc
        if (plop.create_document("hellosign.pdf", optlist) == -1):
            raise PLOPException("Error: %s: %s" % (plop.get_apiname(),
                                                     plop.get_errmsg()))

        # close the input document
        plop.close_document(doc, "")


        # delete PVF file containing input document
        if (plop.delete_pvf("/pvf/hello.pdf") == -1):
            raise PLOPException("Error: %s" % plopget_errmsg())

    except PLOPException as ex:
	print("PDFlib PLOP exception occurred:")
	print("[%d] %s: %s" % (ex.errnum, ex.apiname, ex.errmsg))

except Exception as ex:
    print(ex)

finally:
    if plop:
        plop.delete()
    if p:
        p.delete()
